#! /usr/bin/env python

import sys

import pybindgen
from pybindgen import ReturnValue, Parameter, Module, CppClass, CppMethod, CppConstructor, FileCodeSink

def my_module_gen(out_file):

    mod = Module('h')
    mod.add_include('"h.h"')

    H = mod.add_class('H')
    H.add_constructor([])
    H.add_method('Do', None, [])

    Inner = mod.add_class('Inner', outer_class=H)
    Inner.add_constructor([])
    Inner.add_method('Do', None, [])

    MostInner = mod.add_class('MostInner', outer_class=Inner)
    MostInner.add_constructor([])
    MostInner.add_method('Do', None, [])

    mod.generate(FileCodeSink(out_file))

if __name__ == '__main__':
    my_module_gen(sys.stdout)



#! /usr/bin/env python
from __future__ import unicode_literals, print_function

import sys
import os.path

import pybindgen
from pybindgen.module import MultiSectionFactory
from pybindgen import (FileCodeSink)

import foomodulegen_split
import foomodulegen_common


class MyMultiSectionFactory(MultiSectionFactory):

    def __init__(self, main_file_name):
        self.main_file_name = main_file_name
        self.main_sink = FileCodeSink(open(main_file_name, "wt"))
        self.header_name = "foomodule4.h"
        header_file_name = os.path.join(os.path.dirname(self.main_file_name), self.header_name)
        self.header_sink = FileCodeSink(open(header_file_name, "wt"))
        self.section_sinks = {}

    def get_section_code_sink(self, section_name):
        if section_name == '__main__':
            return self.main_sink
        try:
            return self.section_sinks[section_name]
        except KeyError:
            file_name = os.path.join(os.path.dirname(self.main_file_name), "%s.cc" % section_name)
            sink = FileCodeSink(open(file_name, "wt"))
            self.section_sinks[section_name] = sink
            return sink

    def get_main_code_sink(self):
        return self.main_sink

    def get_common_header_code_sink(self):
        return self.header_sink

    def get_common_header_include(self):
        return '"%s"' % self.header_name

    def close(self):
        self.header_sink.file.close()
        self.main_sink.file.close()
        for sink in self.section_sinks.values():
            sink.file.close()

def my_module_gen():
    out = MyMultiSectionFactory(sys.argv[1])
    root_module = foomodulegen_split.module_init()
    root_module.add_exception('exception', foreign_cpp_namespace='std', message_rvalue='%(EXC)s.what()')
    foomodulegen_common.customize_module_pre(root_module)

    foomodulegen_split.register_types(root_module)
    foomodulegen_split.register_methods(root_module)
    foomodulegen_split.register_functions(root_module)
    foomodulegen_common.customize_module(root_module)

    root_module.generate(out)

    out.close()



if __name__ == '__main__':
    try:
        import cProfile as profile
    except ImportError:
        my_module_gen()
    else:
        print("** running under profiler", file=sys.stderr)
        profile.run('my_module_gen()', 'foomodulegen4.pstat')




#! /usr/bin/env python

import sys

import pybindgen
from pybindgen import ReturnValue, Parameter, FileCodeSink
from pybindgen.module import Module
from pybindgen.function import Function

def my_module_gen(out_file):

    mod = Module('a')
    mod.add_include('"a.h"')

    mod.add_function('ADoA', None, [])
    mod.add_function('ADoB', None, [Parameter.new('uint32_t', 'b')])
    mod.add_function('ADoC', ReturnValue.new('uint32_t'), [])

    mod.generate(FileCodeSink(out_file) )

if __name__ == '__main__':
    my_module_gen(sys.stdout)



"""
The class that generates code to keep track of existing python
wrappers for a given root class.
"""

from pybindgen.typehandlers.base import NotSupportedError


class WrapperRegistry(object):
    """
    Abstract base class for wrapepr registries.
    """

    def __init__(self, base_name):
        self.base_name = base_name

    def generate_forward_declarations(self, code_sink, module):
        raise NotImplementedError

    def generate(self, code_sink, module, import_from_module):
        raise NotImplementedError

    def write_register_new_wrapper(self, code_block, wrapper_lvalue, object_rvalue):
        raise NotImplementedError
        
    def write_lookup_wrapper(self, code_block, wrapper_type, wrapper_lvalue, object_rvalue):
        raise NotImplementedError

    def write_unregister_wrapper(self, code_block, wrapper_lvalue, object_rvalue):
        raise NotImplementedError

    
class NullWrapperRegistry(WrapperRegistry):
    """
    A 'null' wrapper registry class.  It produces no code, and does
    not guarantee that more than one wrapper cannot be created for
    each object.  Use this class to disable wrapper registries entirely.
    """

    def __init__(self, base_name):
        super(NullWrapperRegistry, self).__init__(base_name)

    def generate_forward_declarations(self, code_sink, module, import_from_module):
        pass

    def generate(self, code_sink, module):
        pass
    def generate_import(self, code_sink, module, import_from_module):
        pass

    def write_register_new_wrapper(self, code_block, wrapper_lvalue, object_rvalue):
        pass
        
    def write_lookup_wrapper(self, code_block, wrapper_type, wrapper_lvalue, object_rvalue):
        raise NotSupportedError

    def write_unregister_wrapper(self, code_block, wrapper_lvalue, object_rvalue):
        pass


class StdMapWrapperRegistry(WrapperRegistry):
    """
    A wrapper registry that uses std::map as implementation.  Do not
    use this if generating pure C wrapping code, else the code will
    not compile.
    """


    def __init__(self, base_name):
        super(StdMapWrapperRegistry, self).__init__(base_name)
        self.map_name = "%s_wrapper_registry" % base_name

    def generate_forward_declarations(self, code_sink, module, import_from_module):
        module.add_include("<map>")
        module.add_include("<iostream>")
        #code_sink.writeln("#include <map>")
        #code_sink.writeln("#include <iostream>")
        if import_from_module:
            code_sink.writeln("extern std::map<void*, PyObject*> *_%s;" % self.map_name)
            code_sink.writeln("#define %s (*_%s)" % (self.map_name, self.map_name))
        else:
            code_sink.writeln("extern std::map<void*, PyObject*> %s;" % self.map_name)

    def generate(self, code_sink, module):
        code_sink.writeln("std::map<void*, PyObject*> %s;" % self.map_name)
        # register the map in the module namespace
        module.after_init.write_code("PyModule_AddObject(m, (char *) \"_%s\", PyCObject_FromVoidPtr(&%s, NULL));"
                                     % (self.map_name, self.map_name))

    def generate_import(self, code_sink, code_block, module_pyobj_var):
        code_sink.writeln("std::map<void*, PyObject*> *_%s;" % self.map_name)
        code_block.write_code("PyObject *_cobj = PyObject_GetAttrString(%s, (char*) \"_%s\");"
                              % (module_pyobj_var, self.map_name))
        code_block.write_code("if (_cobj == NULL) {\n"
                              "    _%(MAP)s = NULL;\n"
                              "    PyErr_Clear();\n"
                              "} else {\n"
                              "    _%(MAP)s = reinterpret_cast< std::map<void*, PyObject*> *> (PyCObject_AsVoidPtr (_cobj));\n"
                              "    Py_DECREF(_cobj);\n"
                              "}"
                              % dict(MAP=self.map_name))

    def write_register_new_wrapper(self, code_block, wrapper_lvalue, object_rvalue):
        code_block.write_code("%s[(void *) %s] = (PyObject *) %s;" % (self.map_name, object_rvalue, wrapper_lvalue))
        #code_block.write_code('std::cerr << "Register Wrapper: obj=" <<(void *) %s << ", wrapper=" << %s << std::endl;'
        #                      % (object_rvalue, wrapper_lvalue))
        
    def write_lookup_wrapper(self, code_block, wrapper_type, wrapper_lvalue, object_rvalue):
        iterator = code_block.declare_variable("std::map<void*, PyObject*>::const_iterator", "wrapper_lookup_iter")
        #code_block.write_code('std::cerr << "Lookup Wrapper: obj=" <<(void *) %s << " map size: " << %s.size() << std::endl;'
        #                      % (object_rvalue, self.map_name))
        code_block.write_code("%s = %s.find((void *) %s);" % (iterator, self.map_name, object_rvalue))
        code_block.write_code("if (%(ITER)s == %(MAP)s.end()) {\n"
                              "    %(WRAPPER)s = NULL;\n"
                              "} else {\n"
                              "    %(WRAPPER)s = (%(TYPE)s *) %(ITER)s->second;\n"
                              "    Py_INCREF(%(WRAPPER)s);\n"
                              "}\n"
                              % dict(ITER=iterator, MAP=self.map_name, WRAPPER=wrapper_lvalue, TYPE=wrapper_type))
        
    def write_unregister_wrapper(self, code_block, wrapper_lvalue, object_rvalue):
        #code_block.write_code('std::cerr << "Erase Wrapper: obj=" <<(void *) %s << std::endl;'
        #                      % (object_rvalue))
        iterator = code_block.declare_variable("std::map<void*, PyObject*>::iterator", "wrapper_lookup_iter")
        code_block.write_code("%(ITER)s = %(MAP)s.find((void *) %(OBJECT_VALUE)s);\n"
                              "if (%(ITER)s != %(MAP)s.end()) {\n"
                              "    %(MAP)s.erase(%(ITER)s);\n"
                              "}\n"
                              % dict(ITER=iterator, MAP=self.map_name, WRAPPER=wrapper_lvalue, OBJECT_VALUE=object_rvalue))





from pybindgen import settings
from pybindgen import utils


class CppException(object):
    def __init__(self, name, parent=None, outer_class=None, custom_name=None,
                 foreign_cpp_namespace=None, message_rvalue=None, is_standard_error=False):
        """
        :param name: exception class name
        :param parent: optional parent class wrapper

        :param custom_name: an alternative name to give to this
               exception class at python-side; if omitted, the name of
               the class in the python module will be the same name as
               the class in C++ (minus namespace).

        :param is_standard_error: if True (default false), throws a
               standard Python Exception with the same name instead
               of creating anew exception. If custom_name is set,
               it uses the standard exception with that name instead.

        :param foreign_cpp_namespace: if set, the class is assumed to
               belong to the given C++ namespace, regardless of the
               C++ namespace of the python module it will be added to.
               For instance, this can be useful to wrap std classes,
               like std::ofstream, without having to create an extra
               python submodule.

        :param message_rvalue: if not None, this parameter is a string
               that contains an rvalue C expression that evaluates to
               the exception message.  The Python % operator will be
               used to substitute %(EXC)s for the caught exception
               variable name.  The rvalue expression must return a
               string of type "char const*", a pointer owned by the
               exception instance.
        """
        self.name = name
        self.full_name = None
        self.parent = parent
        self._module = None
        self.outer_class = outer_class
        self.custom_name = custom_name
        self.mangled_name = None
        self.mangled_full_name = None
        self.pytypestruct = None
        self.foreign_cpp_namespace = foreign_cpp_namespace
        self.message_rvalue = message_rvalue
        self.is_standard_error = is_standard_error

        
    def __repr__(self):
        return "<pybindgen.CppException %r>" % self.full_name

    def write_convert_to_python(self, code_block, variable_name):
        if self.message_rvalue is None:
            code_block.write_code('PyErr_SetNone((PyObject *) %s);' % self.pytypestruct)
        else:
            code_block.write_code('PyErr_SetString((PyObject *) %s, %s);'
                                  % (self.pytypestruct, (self.message_rvalue % dict(EXC=variable_name))))
    
    def get_module(self):
        """Get the Module object this type belongs to"""
        return self._module

    def set_module(self, module):
        """Set the Module object this type belongs to"""
        self._module = module
        self._update_names()

    module = property(get_module, set_module)

    def _update_names(self):
        
        prefix = settings.name_prefix.capitalize()

        if self.outer_class is None:
            if self.foreign_cpp_namespace:
                self.full_name = self.foreign_cpp_namespace + '::' + self.name
            else:
                if self._module.cpp_namespace_prefix:
                    if self._module.cpp_namespace_prefix == '::':
                        self.full_name = '::' + self.name
                    else:
                        self.full_name = self._module.cpp_namespace_prefix + '::' + self.name
                else:
                    self.full_name = self.name
        else:
            assert not self.foreign_cpp_namespace
            self.full_name = '::'.join([self.outer_class.full_name, self.name])

        def make_upper(s):
            if s and s[0].islower():
                return s[0].upper()+s[1:]
            else:
                return s

        def flatten(name):
            "make a name like::This look LikeThis"
            return ''.join([make_upper(utils.mangle_name(s)) for s in name.split('::')])

        self.mangled_name = flatten(self.name)
        self.mangled_full_name = utils.mangle_name(self.full_name)

        if self.is_standard_error:
            self.pytypestruct = "PyExc_%s" % self.python_name
        else:
            self.pytypestruct = "Py%s%s_Type" % (prefix,  self.mangled_full_name)

    def _get_python_name(self):
        if self.custom_name is None:
            class_python_name = self.name
        else:
            class_python_name = self.custom_name
        return class_python_name

    python_name = property(_get_python_name)

    def _get_python_full_name(self):
        if self.outer_class is None:
            mod_path = self._module.get_module_path()
            mod_path.append(self.python_name)
            return '.'.join(mod_path)
        else:
            return '%s.%s' % (self.outer_class.pytype.slots['tp_name'], self.python_name)
    python_full_name = property(_get_python_full_name)


    def generate_forward_declarations(self, code_sink, dummy_module):
        if self.is_standard_error:
            return

        code_sink.writeln()
        code_sink.writeln('extern PyTypeObject *%s;' % (self.pytypestruct,))
        code_sink.writeln()


    def generate(self, code_sink, module, docstring=None):
        """Generates the class to a code sink"""

        # skip generation if translating to a standard Exception
        if self.is_standard_error:
            return

        code_sink.writeln('PyTypeObject *%s;' % (self.pytypestruct,))
        ## --- register the class type in the module ---
        module.after_init.write_code("/* Register the '%s' exception */" % self.full_name)
        if self.parent is None:
            parent = 'NULL'
        else:
            parent = "(PyObject*) "+self.parent.pytypestruct
        module.after_init.write_error_check('(%s = (PyTypeObject*) PyErr_NewException((char*)"%s", %s, NULL)) == NULL'
                                            % (self.pytypestruct, self.python_full_name, parent))
        if docstring:
            module.after_init.write_code("%s->tp_doc = (char*)\"%s\";" % (self.pytypestruct, docstring))

        if self.outer_class is None:
            module.after_init.write_code(
                'Py_INCREF((PyObject *) %s);\n'
                'PyModule_AddObject(m, (char *) \"%s\", (PyObject *) %s);' % (
                self.pytypestruct, self.python_name, self.pytypestruct))
        else:
            module.after_init.write_code(
                'Py_INCREF((PyObject *) %s);\n'
                'PyDict_SetItemString((PyObject*) %s.tp_dict, (char *) \"%s\", (PyObject *) %s);' % (
                self.pytypestruct, self.outer_class.pytypestruct, self.python_name, self.pytypestruct))



#!/usr/bin/env python
from setuptools import setup

with open('README.rst') as file_:
    LONG_DESCRIPTION = file_.read()


setup(name='PyBindGen',
      use_scm_version={"version_scheme": "post-release",
                       "write_to": "pybindgen/version.py"},
      setup_requires=['setuptools_scm'],
      description='Python Bindings Generator',
      author='Gustavo Carneiro',
      author_email='gjcarneiro@gmail.com',
      url='https://launchpad.net/pybindgen',
      packages=['pybindgen',
                'pybindgen.typehandlers',
                'pybindgen.typehandlers.ctypeparser',
                ],
      long_description=LONG_DESCRIPTION,
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Build Tools',
          'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 3',
      ],
)



# docstrings not neede here (the type handler doubleerfaces are fully
# documented in base.py) pylint: disable-msg=C0111

from .base import ReturnValue, Parameter, \
     ReverseWrapperBase, ForwardWrapperBase

class DoubleParam(Parameter):

    DIRECTIONS = [Parameter.DIRECTION_IN]
    CTYPES = ['double']

    def convert_c_to_python(self, wrapper):
        assert isinstance(wrapper, ReverseWrapperBase)
        wrapper.build_params.add_parameter('d', [self.value])

    def convert_python_to_c(self, wrapper):
        assert isinstance(wrapper, ForwardWrapperBase)
        name = wrapper.declarations.declare_variable(self.ctype_no_const, self.name, self.default_value)
        assert "const" not in self.ctype_no_const
        wrapper.parse_params.add_parameter('d', ['&'+name], self.value, optional=bool(self.default_value))
        wrapper.call_params.append(name)


class DoubleReturn(ReturnValue):

    CTYPES = ['double']

    def get_c_error_return(self):
        return "return 0;"
    
    def convert_python_to_c(self, wrapper):
        wrapper.parse_params.add_parameter("d", ["&"+self.value], prepend=True)

    def convert_c_to_python(self, wrapper):
        wrapper.build_params.add_parameter("d", [self.value], prepend=True)


class DoublePtrParam(Parameter):

    DIRECTIONS = [Parameter.DIRECTION_IN, Parameter.DIRECTION_OUT,
                  Parameter.DIRECTION_IN|Parameter.DIRECTION_OUT]
    CTYPES = ['double*']
    
    def convert_c_to_python(self, wrapper):
        if self.direction & self.DIRECTION_IN:
            wrapper.build_params.add_parameter('d', ['*'+self.value])
        if self.direction & self.DIRECTION_OUT:
            wrapper.parse_params.add_parameter("d", [self.value], self.name)

    def convert_python_to_c(self, wrapper):
        base_ctype = self.type_traits.target or self.ctype_no_const
        name = wrapper.declarations.declare_variable(str(base_ctype), self.name + "xxxxx")
        wrapper.call_params.append('&'+name)
        if self.direction & self.DIRECTION_IN:
            wrapper.parse_params.add_parameter('d', ['&'+name], self.name)
        if self.direction & self.DIRECTION_OUT:
            wrapper.build_params.add_parameter("d", [name])
        


class DoubleRefParam(Parameter):

    DIRECTIONS = [Parameter.DIRECTION_IN, Parameter.DIRECTION_OUT,
                  Parameter.DIRECTION_IN|Parameter.DIRECTION_OUT]
    CTYPES = ['double&']
    
    def convert_c_to_python(self, wrapper):
        if self.direction & self.DIRECTION_IN:
            wrapper.build_params.add_parameter('d', [self.value])
        if self.direction & self.DIRECTION_OUT:
            wrapper.parse_params.add_parameter("d", [self.value], self.name)

    def convert_python_to_c(self, wrapper):
        #assert self.ctype == 'double&'
        name = wrapper.declarations.declare_variable('double', self.name)
        wrapper.call_params.append(name)
        if self.direction & self.DIRECTION_IN:
            wrapper.parse_params.add_parameter('d', ['&'+name], self.name)
        if self.direction & self.DIRECTION_OUT:
            wrapper.build_params.add_parameter("d", [name])



from pybindgen.typehandlers.ctypeparser import tokenizer


MODIFIERS = ['const', 'volatile'] # XXX: are there others?

try:
    set
except NameError:
    from sets import Set as set


class CType(object):
    """
    A L{CType} represents a C/C++ type as a list of items.  Generally
    the items are L{Token}s, but some times they can be other
    L{CType}s (arguments of templated types, function pointer name and parameters).
    """
    __slots__ = 'tokens'
    def __init__(self, tokens=None):
        if tokens is None:
            self.tokens = []
        else:
            self.tokens = tokens

    def clone(self):
        return CType(list(self.tokens))

    def reorder_modifiers(self):
        """
        Reoder const modifiers, as rightward as possible without
        changing the meaning of the type.  I.e., move modifiers to the
        right until a * or & is found."""
        for modifier in MODIFIERS:
            self._reorder_modifier(modifier)

    def _reorder_modifier(self, modifier):
        tokens_moved = []
        while 1:
            reordered = False
            for token_i, token in enumerate(self.tokens):
                if isinstance(token, CType):
                    continue
                if token.name == modifier and token not in tokens_moved:
                    ## Reorder the token.  Note: we are mutating the
                    ## list we are iterating over, but it's ok because
                    ## we'll break the for unconditionally next.

                    self.tokens.pop(token_i)
                    
                    for new_pos in range(token_i, len(self.tokens)):
                        other_token = self.tokens[new_pos]
                        if isinstance(other_token, CType):
                            continue
                        if other_token.name in ['*', '&']:
                            self.tokens.insert(new_pos, token)
                            break
                    else:
                        self.tokens.append(token)
                        new_pos = -1

                    tokens_moved.append(token)
                    reordered = True
                    break
            if not reordered:
                break

    def remove_modifiers(self):
        """
        Remove modifiers from the toplevel type.  Return a set of modifiers removed.
        """
        retval = set()
        for modifier in MODIFIERS:
            if self._remove_modifier(modifier):
                retval.add(modifier)
        return retval

    def _remove_modifier(self, modifier):
        changed = True
        removed = False
        while changed:
            changed = False
            for token_i, token in enumerate(self.tokens):
                if isinstance(token, CType):
                    continue
                if token.name == modifier:
                    del self.tokens[token_i]
                    changed = True
                    removed = True
                    break
        return removed

    def remove_outer_modifier(self, modifier):
        """
        Remove the given modifier from the type, but only from the
        outer part and only until a first * or & is found, from right
        to left.
        """
        for token_i in range(len(self.tokens)-1, -1, -1):
            token = self.tokens[token_i]
            if isinstance(token, CType):
                continue
            if token.name == modifier:
                del self.tokens[token_i]
                return True
        return False

    def __str__(self):
        l = []
        first = True
        for token in self.tokens:
            if isinstance(token, tokenizer.Token):
                if token.name in "<,":
                    l.append(token.name)
                else:
                    if first:
                        l.append(token.name)
                    else:
                        l.append(' ' + token.name)
            else:
                assert isinstance(token, CType)
                if first:
                    l.append(str(token))
                else:
                    l.append(' ' + str(token))
            first = False
        return ''.join(l)


def _parse_type_recursive(tokens):
    ctype = CType()
    while tokens:
        token = tokens.pop(0)
        if token.name.startswith('::'):
            token.name = token.name[2:]
        if token.token_type == tokenizer.SYNTAX:
            if token.name in [',', '>', ')']:
                ctype.reorder_modifiers()
                return ctype, token
            elif token.name in ['<', '(']:
                ctype.tokens.append(token)
                while 1:
                    nested_ctype, last_token = _parse_type_recursive(tokens)
                    ctype.tokens.append(nested_ctype)
                    ctype.tokens.append(last_token)
                    assert token.token_type == tokenizer.SYNTAX
                    if last_token.name == ',':
                        continue
                    elif last_token.name in ['>', ')']:
                        break
                    else:
                        assert False, ("last_token invalid: %s" % last_token)
            else:
                ctype.tokens.append(token)
        else:
            ctype.tokens.append(token)
    ctype.reorder_modifiers()
    return ctype, None


def parse_type(type_string):
    """
    Parse a C type string.

    :param type_string: C type expression
    :returns: a L{CType} object representing the type
    """
    tokens = list(tokenizer.GetTokens(type_string + '\n'))
    ctype, last_token = _parse_type_recursive(tokens)
    assert last_token is None
    return ctype

def normalize_type_string(type_string):
    """
    Return a type string in a canonical format, with deterministic
    placement of modifiers and spacing.  Useful to make sure two type
    strings match regardless of small variations of representation
    that do not change the meaning.

    :param type_string: C type expression
    :returns: another string representing the same C type but in a canonical format

    >>> normalize_type_string('char *')
    'char *'
    >>> normalize_type_string('const foo::bar<const char*, zbr&>*')
    'foo::bar< char const *, zbr & > const *'
    >>> normalize_type_string('const ::bar*')
    'bar const *'
    >>> normalize_type_string('const char*const')
    'char const * const'
    >>> normalize_type_string('const char*const*const')
    'char const * const * const'
    >>> normalize_type_string('const std::map<std::string, void (*) (int, std::vector<zbr>) >')
    'std::map< std::string, void ( * ) ( int, std::vector< zbr > ) > const'
    """
    ctype = parse_type(type_string)
    return str(ctype)


class TypeTraits(object):
    """
    Parse a C type and gather some interesting properties.

    @ivar ctype: the original unmodified type (a L{CType} object, apply str() to obtain a type string).

    @ivar ctype_no_modifiers: the type with all modifiers (const, volatile, ...) removed (except from template arguments)

    @ivar type_is_const: True if the outermost type is const

    @ivar type_is_reference: True if the outermost type is a reference

    @ivar type_is_pointer:  True if the outermost type is a pointer

    @ivar target_is_const: True if the type is pointer or reference and the target is const

    @ivar target: if this is a pointer or reference type, a L{CType}
    representing the target, without modifiers.  If not pointer or
    reference, it is None.

    >>> t = TypeTraits("int")
    >>> print repr(str(t.ctype))
    'int'
    >>> print repr(str(t.ctype_no_modifiers))
    'int'
    >>> t.type_is_const
    False
    >>> t.type_is_pointer
    False
    >>> t.type_is_reference
    False
    >>> t.target is None
    True

    >>> t = TypeTraits("const int * const")
    >>> print repr(str(t.ctype))
    'int const * const'
    >>> print repr(str(t.ctype_no_modifiers))
    'int *'
    >>> print repr(str(t.ctype_no_const))
    'int const *'
    >>> t.type_is_const
    True
    >>> t.type_is_pointer
    True
    >>> t.type_is_reference
    False
    >>> t.target is None
    False
    >>> print repr(str(t.target))
    'int'
    >>> t.target_is_const
    True

    >>> t = TypeTraits("int * const")
    >>> print repr(str(t.ctype))
    'int * const'
    >>> print repr(str(t.ctype_no_modifiers))
    'int *'
    >>> print repr(str(t.ctype_no_const))
    'int *'
    >>> t.type_is_const
    True
    >>> t.type_is_pointer
    True
    >>> t.type_is_reference
    False
    >>> t.target is None
    False
    >>> print repr(str(t.target))
    'int'
    >>> t.target_is_const
    False

    >>> t = TypeTraits("const char *")
    >>> print repr(str(t.ctype))
    'char const *'
    >>> print repr(str(t.ctype_no_modifiers))
    'char *'
    >>> print repr(str(t.ctype_no_const))
    'char const *'
    >>> t.type_is_const
    False
    >>> t.type_is_pointer
    True
    >>> t.type_is_reference
    False
    >>> t.target is None
    False
    >>> print repr(str(t.target))
    'char'
    >>> t.target_is_const
    True

    >>> t = TypeTraits("char *")
    >>> print repr(str(t.ctype))
    'char *'
    >>> t.make_const()
    >>> print repr(str(t.ctype))
    'char * const'
    >>> t.make_target_const()
    >>> print repr(str(t.ctype))
    'char const * const'

    """

    def __init__(self, ctype):
        self.ctype = parse_type(ctype)
        self.ctype_no_modifiers = self.ctype.clone()
        self.ctype_no_modifiers.remove_modifiers()
        self.ctype_no_const = self.ctype.clone()
        tokens = list(self.ctype.tokens)
        tokens.reverse()
        ptr_ref_level = 0
        self.type_is_const = False
        self.type_is_reference = False
        self.type_is_pointer = False
        self.target_is_const = False
        self.target = None
        target_pos = None
        for pos, token in enumerate(tokens):
            if isinstance(token, CType):
                continue
            if token.name == 'const':
                if ptr_ref_level == 0:
                    self.type_is_const = True
                    const_removed = self.ctype_no_const.remove_outer_modifier('const')
                    assert const_removed
                elif ptr_ref_level == 1:
                    self.target_is_const = True
            elif token.name == '*':
                if ptr_ref_level == 0:
                    self.type_is_pointer = True
                    target_pos = pos + 1
                ptr_ref_level += 1
            elif token.name == '&':
                if ptr_ref_level == 0:
                    self.type_is_reference = True
                    target_pos = pos + 1
                ptr_ref_level += 1
        if target_pos is not None:
            target_tokens = tokens[target_pos:]
            target_tokens.reverse()
            self.target = CType(target_tokens)
            self.target.remove_modifiers()
        self.ctype_no_const_no_ref = self.ctype_no_const.clone()
        self.ctype_no_const_no_ref.remove_outer_modifier("&")

    def make_const(self):
        """
        Add a const modifier to the type.  Has no effect if the type is already const.
        """
        if self.type_is_const:
            return
        self.type_is_const = True
        self.ctype.tokens.append(tokenizer.Token(tokenizer.NAME, "const", None, None))

    def make_target_const(self):
        """
        Add a const modifier to the type target.  Has no effect if the type target is already const.
        """
        assert self.type_is_pointer or self.type_is_reference
        if self.target_is_const:
            return
        self.target_is_const = True
        for tokens in self.ctype.tokens, self.ctype_no_const.tokens:
            for token_i in range(len(tokens)-1, -1, -1):
                token = tokens[token_i]
                if isinstance(token, CType):
                    continue
                elif token.name in ['*', '&']:
                    tokens.insert(token_i, tokenizer.Token(tokenizer.NAME, "const", None, None))
                    break



"""
The class PyTypeObject generates a PyTypeObject structure contents.
"""

class PyTypeObject(object):
    TEMPLATE = (
        'PyTypeObject %(typestruct)s = {\n'
        '    PyVarObject_HEAD_INIT(NULL, 0)\n'
        '    (char *) "%(tp_name)s",            /* tp_name */\n'
        '    %(tp_basicsize)s,                  /* tp_basicsize */\n'
        '    0,                                 /* tp_itemsize */\n'
        '    /* methods */\n'
        '    (destructor)%(tp_dealloc)s,        /* tp_dealloc */\n'
        '    (printfunc)0,                      /* tp_print */\n'
        '    (getattrfunc)%(tp_getattr)s,       /* tp_getattr */\n'
        '    (setattrfunc)%(tp_setattr)s,       /* tp_setattr */\n'
        '#if PY_MAJOR_VERSION >= 3\n'
        '    NULL, \n'
        '#else\n'
        '    (cmpfunc)%(tp_compare)s,           /* tp_compare */\n'
        '#endif\n'
        '    (reprfunc)%(tp_repr)s,             /* tp_repr */\n'
        '    (PyNumberMethods*)%(tp_as_number)s,     /* tp_as_number */\n'
        '    (PySequenceMethods*)%(tp_as_sequence)s, /* tp_as_sequence */\n'
        '    (PyMappingMethods*)%(tp_as_mapping)s,   /* tp_as_mapping */\n'
        '    (hashfunc)%(tp_hash)s,             /* tp_hash */\n'
        '    (ternaryfunc)%(tp_call)s,          /* tp_call */\n'
        '    (reprfunc)%(tp_str)s,              /* tp_str */\n'
        '    (getattrofunc)%(tp_getattro)s,     /* tp_getattro */\n'
        '    (setattrofunc)%(tp_setattro)s,     /* tp_setattro */\n'
        '    (PyBufferProcs*)%(tp_as_buffer)s,  /* tp_as_buffer */\n'
        '    %(tp_flags)s,                      /* tp_flags */\n'
        '    %(tp_doc)s,                        /* Documentation string */\n'
        '    (traverseproc)%(tp_traverse)s,     /* tp_traverse */\n'
        '    (inquiry)%(tp_clear)s,             /* tp_clear */\n'
        '    (richcmpfunc)%(tp_richcompare)s,   /* tp_richcompare */\n'
        '    %(tp_weaklistoffset)s,             /* tp_weaklistoffset */\n'
        '    (getiterfunc)%(tp_iter)s,          /* tp_iter */\n'
        '    (iternextfunc)%(tp_iternext)s,     /* tp_iternext */\n'
        '    (struct PyMethodDef*)%(tp_methods)s, /* tp_methods */\n'
        '    (struct PyMemberDef*)0,              /* tp_members */\n'
        '    %(tp_getset)s,                     /* tp_getset */\n'
        '    NULL,                              /* tp_base */\n'
        '    NULL,                              /* tp_dict */\n'
        '    (descrgetfunc)%(tp_descr_get)s,    /* tp_descr_get */\n'
        '    (descrsetfunc)%(tp_descr_set)s,    /* tp_descr_set */\n'
        '    %(tp_dictoffset)s,                 /* tp_dictoffset */\n'
        '    (initproc)%(tp_init)s,             /* tp_init */\n'
        '    (allocfunc)%(tp_alloc)s,           /* tp_alloc */\n'
        '    (newfunc)%(tp_new)s,               /* tp_new */\n'
        '    (freefunc)%(tp_free)s,             /* tp_free */\n'
        '    (inquiry)%(tp_is_gc)s,             /* tp_is_gc */\n'
        '    NULL,                              /* tp_bases */\n'
        '    NULL,                              /* tp_mro */\n'
        '    NULL,                              /* tp_cache */\n'
        '    NULL,                              /* tp_subclasses */\n'
        '    NULL,                              /* tp_weaklist */\n'
        '    (destructor) NULL                  /* tp_del */\n'
        '};\n'
        )

    def __init__(self):
        self.slots = {}

    def generate(self, code_sink):
        """
        Generates the type structure.  All slots are optional except
        'tp_name', 'tp_basicsize', and the pseudo-slot 'typestruct'.
        """

        slots = dict(self.slots)

        slots.setdefault('tp_dealloc', 'NULL')
        slots.setdefault('tp_getattr', 'NULL')
        slots.setdefault('tp_setattr', 'NULL')
        slots.setdefault('tp_compare', 'NULL')
        slots.setdefault('tp_repr', 'NULL')
        slots.setdefault('tp_as_number', 'NULL')
        slots.setdefault('tp_as_sequence', 'NULL')
        slots.setdefault('tp_as_mapping', 'NULL')
        slots.setdefault('tp_hash', 'NULL')
        slots.setdefault('tp_call', 'NULL')
        slots.setdefault('tp_str', 'NULL')
        slots.setdefault('tp_getattro', 'NULL')
        slots.setdefault('tp_setattro', 'NULL')
        slots.setdefault('tp_as_buffer', 'NULL')
        slots.setdefault('tp_flags', 'Py_TPFLAGS_DEFAULT')
        slots.setdefault('tp_doc', 'NULL')
        slots.setdefault('tp_traverse', 'NULL')
        slots.setdefault('tp_clear', 'NULL')
        slots.setdefault('tp_richcompare', 'NULL')
        slots.setdefault('tp_weaklistoffset', '0')
        slots.setdefault('tp_iter', 'NULL')
        slots.setdefault('tp_iternext', 'NULL')
        slots.setdefault('tp_methods', 'NULL')
        slots.setdefault('tp_getset', 'NULL')
        slots.setdefault('tp_descr_get', 'NULL')
        slots.setdefault('tp_descr_set', 'NULL')
        slots.setdefault('tp_dictoffset', '0')
        slots.setdefault('tp_init', 'NULL')
        slots.setdefault('tp_alloc', 'PyType_GenericAlloc')
        slots.setdefault('tp_new', 'PyType_GenericNew')
        slots.setdefault('tp_free', '0')
        slots.setdefault('tp_is_gc', 'NULL')

        code_sink.writeln(self.TEMPLATE % slots)


class PyNumberMethods(object):
    TEMPLATE = (
        'static PyNumberMethods %(variable)s = {\n'
        '#if PY_VERSION_HEX < 0x03000000\n'
        '    (binaryfunc) %(nb_add)s,\n'
        '    (binaryfunc) %(nb_subtract)s,\n'
        '    (binaryfunc) %(nb_multiply)s,\n'
        '    (binaryfunc) %(nb_divide)s,\n'
        '    (binaryfunc) %(nb_remainder)s,\n'
        '    (binaryfunc) %(nb_divmod)s,\n'
        '    (ternaryfunc) %(nb_power)s,\n'
        '    (unaryfunc) %(nb_negative)s,\n'
        '    (unaryfunc) %(nb_positive)s,\n'
        '    (unaryfunc) %(nb_absolute)s,\n'
        '    (inquiry) %(nb_nonzero)s,\n'
        '    (unaryfunc) %(nb_invert)s,\n'
        '    (binaryfunc) %(nb_lshift)s,\n'
        '    (binaryfunc) %(nb_rshift)s,\n'
        '    (binaryfunc) %(nb_and)s,\n'
        '    (binaryfunc) %(nb_xor)s,\n'
        '    (binaryfunc) %(nb_or)s,\n'
        '    (coercion) %(nb_coerce)s,\n'
        '    (unaryfunc) %(nb_int)s,\n'
        '    (unaryfunc) %(nb_long)s,\n'
        '    (unaryfunc) %(nb_float)s,\n'
        '    (unaryfunc) %(nb_oct)s,\n'
        '    (unaryfunc) %(nb_hex)s,\n'
        '    /* Added in release 2.0 */\n'
        '    (binaryfunc) %(nb_inplace_add)s,\n'
        '    (binaryfunc) %(nb_inplace_subtract)s,\n'
        '    (binaryfunc) %(nb_inplace_multiply)s,\n'
        '    (binaryfunc) %(nb_inplace_divide)s,\n'
        '    (binaryfunc) %(nb_inplace_remainder)s,\n'
        '    (ternaryfunc) %(nb_inplace_power)s,\n'
        '    (binaryfunc) %(nb_inplace_lshift)s,\n'
        '    (binaryfunc) %(nb_inplace_rshift)s,\n'
        '    (binaryfunc) %(nb_inplace_and)s,\n'
        '    (binaryfunc) %(nb_inplace_xor)s,\n'
        '    (binaryfunc) %(nb_inplace_or)s,\n'
        '\n'
        '    /* Added in release 2.2 */\n'
        '    /* The following require the Py_TPFLAGS_HAVE_CLASS flag */\n'
        '    (binaryfunc) %(nb_floor_divide)s,\n'
        '    (binaryfunc) %(nb_true_divide)s,\n'
        '    (binaryfunc) %(nb_inplace_floor_divide)s,\n'
        '    (binaryfunc) %(nb_inplace_true_divide)s,\n'
        '\n'
        '#if PY_VERSION_HEX >= 0x020500F0\n'
        '    /* Added in release 2.5 */\n'
        '    (unaryfunc) %(nb_index)s,\n'
        '\n'
        '#endif\n'

        '#else /* Python 3 changed this structure a lot */\n'

        '(binaryfunc) %(nb_add)s,\n'
        '(binaryfunc) %(nb_subtract)s,\n'
        '(binaryfunc) %(nb_multiply)s,\n'
        '(binaryfunc) %(nb_remainder)s,\n'
        '(binaryfunc) %(nb_divmod)s,\n'
        '(ternaryfunc) %(nb_power)s,\n'
        '(unaryfunc) %(nb_negative)s,\n'
        '(unaryfunc) %(nb_positive)s,\n'
        '(unaryfunc) %(nb_absolute)s,\n'
        '(inquiry) %(nb_bool)s,\n'
        '(unaryfunc) %(nb_invert)s,\n'
        '(binaryfunc) %(nb_lshift)s,\n'
        '(binaryfunc) %(nb_rshift)s,\n'
        '(binaryfunc) %(nb_and)s,\n'
        '(binaryfunc) %(nb_xor)s,\n'
        '(binaryfunc) %(nb_or)s,\n'
        '(unaryfunc) %(nb_int)s,\n'
        'NULL,\n'
        '(unaryfunc) %(nb_float)s,\n'
        '\n'
        '(binaryfunc) %(nb_inplace_add)s,\n'
        '(binaryfunc) %(nb_inplace_subtract)s,\n'
        '(binaryfunc) %(nb_inplace_multiply)s,\n'
        '(binaryfunc) %(nb_inplace_remainder)s,\n'
        '(ternaryfunc) %(nb_inplace_power)s,\n'
        '(binaryfunc) %(nb_inplace_lshift)s,\n'
        '(binaryfunc) %(nb_inplace_rshift)s,\n'
        '(binaryfunc) %(nb_inplace_and)s,\n'
        '(binaryfunc) %(nb_inplace_xor)s,\n'
        '(binaryfunc) %(nb_inplace_or)s,\n'
        '\n'
        '(binaryfunc) %(nb_floor_divide)s,\n'
        '(binaryfunc) %(nb_divide)s,\n'
        '(binaryfunc) %(nb_inplace_floor_divide)s,\n'
        '(binaryfunc) %(nb_inplace_divide)s,\n'
        '\n'
        '(unaryfunc) %(nb_index)s,\n'
        '#endif\n'

        '};\n'
        )

    def __init__(self):
        self.slots = {}

    def generate(self, code_sink):
        """
        Generates the structure.  All slots are optional except 'variable'.
        """

        slots = dict(self.slots)

        slots.setdefault('nb_add', 'NULL')
        slots.setdefault('nb_bool', 'NULL')
        slots.setdefault('nb_subtract', 'NULL')
        slots.setdefault('nb_multiply', 'NULL')
        slots.setdefault('nb_divide', 'NULL')
        slots.setdefault('nb_remainder', 'NULL')
        slots.setdefault('nb_divmod', 'NULL')
        slots.setdefault('nb_power', 'NULL')
        slots.setdefault('nb_negative', 'NULL')
        slots.setdefault('nb_positive', 'NULL')
        slots.setdefault('nb_absolute', 'NULL')
        slots.setdefault('nb_nonzero', 'NULL')
        slots.setdefault('nb_invert', 'NULL')
        slots.setdefault('nb_lshift', 'NULL')
        slots.setdefault('nb_rshift', 'NULL')
        slots.setdefault('nb_and', 'NULL')
        slots.setdefault('nb_xor', 'NULL')
        slots.setdefault('nb_or', 'NULL')
        slots.setdefault('nb_coerce', 'NULL')
        slots.setdefault('nb_int', 'NULL')
        slots.setdefault('nb_long', 'NULL')
        slots.setdefault('nb_float', 'NULL')
        slots.setdefault('nb_oct', 'NULL')
        slots.setdefault('nb_hex', 'NULL')
        slots.setdefault('nb_inplace_add', 'NULL')
        slots.setdefault('nb_inplace_subtract', 'NULL')
        slots.setdefault('nb_inplace_multiply', 'NULL')
        slots.setdefault('nb_inplace_divide', 'NULL')
        slots.setdefault('nb_inplace_remainder', 'NULL')
        slots.setdefault('nb_inplace_power', 'NULL')
        slots.setdefault('nb_inplace_lshift', 'NULL')
        slots.setdefault('nb_inplace_rshift', 'NULL')
        slots.setdefault('nb_inplace_and', 'NULL')
        slots.setdefault('nb_inplace_xor', 'NULL')
        slots.setdefault('nb_inplace_or', 'NULL')
        slots.setdefault('nb_floor_divide', 'NULL')
        slots.setdefault('nb_true_divide', 'NULL')
        slots.setdefault('nb_inplace_floor_divide', 'NULL')
        slots.setdefault('nb_inplace_true_divide', 'NULL')
        slots.setdefault('nb_index', 'NULL')

        code_sink.writeln(self.TEMPLATE % slots)

class PySequenceMethods(object):
    TEMPLATE = '''
static PySequenceMethods %(variable)s = {
    (lenfunc) %(sq_length)s,
    (binaryfunc) %(sq_concat)s,
    (ssizeargfunc) %(sq_repeat)s,
    (ssizeargfunc) %(sq_item)s,
#if PY_MAJOR_VERSION < 3
    (ssizessizeargfunc) %(sq_slice)s,
#else
    NULL,
#endif
    (ssizeobjargproc) %(sq_ass_item)s,
#if PY_MAJOR_VERSION < 3
    (ssizessizeobjargproc) %(sq_ass_slice)s,
#else
    NULL,
#endif
    (objobjproc) %(sq_contains)s,
    /* Added in release 2.0 */
    (binaryfunc) %(sq_inplace_concat)s,
    (ssizeargfunc) %(sq_inplace_repeat)s,
};

'''

    FUNCTION_TEMPLATES = {
        # __len__
        "sq_length" : '''
static Py_ssize_t
%(wrapper_name)s (%(py_struct)s *py_self)
{
    PyObject *py_result;
    Py_ssize_t result;

    py_result = %(method_name)s(py_self);
    if (py_result == NULL) {
        PyErr_SetString(PyExc_RuntimeError, "Unknown error in attempting to determine __len__.");
        Py_XDECREF(py_result);
        return -1;
    }
    result = PyLong_AsSsize_t(py_result);
    Py_DECREF(py_result);
    return result;
}

''',

        # __len__
        # This hacky version is necessary 'cause if we're calling a function rather than a method
        # or an overloaded wrapper the args parameter gets tacked into the call sequence.
        "sq_length_ARGS" : '''
static Py_ssize_t
%(wrapper_name)s (%(py_struct)s *py_self)
{
    PyObject *py_result;
    PyObject *args;
    Py_ssize_t result;

    args = PyTuple_New (0);
    py_result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    if (py_result == NULL) {
        PyErr_SetString(PyExc_RuntimeError, "Unknown error in attempting to determine __len__.");
        Py_XDECREF(py_result);
        return -1;
    }
    result = PyLong_AsSsize_t(py_result);
    Py_DECREF(py_result);
    return result;
}

''',

        # __add__ (concatenation)
        "sq_concat" : '''
static PyObject*
%(wrapper_name)s (%(py_struct)s *py_self, %(py_struct)s *py_rhs)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(O)", py_rhs);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    return result;
}

''',

        # __mul__ (repeat)
        "sq_repeat" : '''
static PyObject*
%(wrapper_name)s (%(py_struct)s *py_self, Py_ssize_t py_i)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(i)", py_i);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    return result;
}

''',

        # __getitem__
        "sq_item" : '''
static PyObject*
%(wrapper_name)s (%(py_struct)s *py_self, Py_ssize_t py_i)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(i)", py_i);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    if (PyErr_ExceptionMatches(PyExc_IndexError) ||
        PyErr_ExceptionMatches(PyExc_StopIteration)) {
        Py_XDECREF(result);
        return NULL;
    } else {
        return result;
    }
}


''',

        # __getslice__
        "sq_slice" : '''
#if PY_MAJOR_VERSION < 3
static PyObject*
%(wrapper_name)s (%(py_struct)s *py_self, Py_ssize_t py_i1, Py_ssize_t py_i2)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(ii)", py_i1, py_i2);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    if (PyErr_ExceptionMatches(PyExc_IndexError) ||
        PyErr_ExceptionMatches(PyExc_StopIteration)) {
        Py_XDECREF(result);
        return NULL;
    } else {
        return result;
    }
}
#endif
''',

        # __setitem__
        "sq_ass_item" : '''
static int
%(wrapper_name)s (%(py_struct)s *py_self, Py_ssize_t py_i, PyObject *py_val)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(iO)", py_i, py_val);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    if (result == NULL) {
        PyErr_SetString(PyExc_IndexError, "Unknown error trying to set value in container.");
        return -1;
#if PY_MAJOR_VERSION >= 3
    } else if (PyLong_Check(result) == 0) {
#else
    } else if (PyInt_Check(result) == 0) {
#endif
        PyErr_SetString(PyExc_IndexError, "Error trying to set value in container -- wrapped method should return integer status.");
        return -1;
    } else {
#if PY_MAJOR_VERSION >= 3
        int iresult = int(PyLong_AS_LONG(result));
#else
        int iresult = int(PyInt_AS_LONG(result));
#endif
        Py_DECREF(result);
        return iresult;
    }
}

''',

        # __setslice__
        "sq_ass_slice" : '''
#if PY_MAJOR_VERSION < 3
static int
%(wrapper_name)s (%(py_struct)s *py_self, Py_ssize_t py_i1, Py_ssize_t py_i2, %(py_struct)s *py_vals)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(iiO)", py_i1, py_i2, py_vals);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    if (result == NULL) {
        PyErr_SetString(PyExc_RuntimeError, "Unknown error trying to set slice in container.");
        return -1;
    } else if (PyInt_Check(result) == 0) {
        PyErr_SetString(PyExc_RuntimeError, "Error trying to set slice in container -- wrapped method should return integer status.");
        return -1;
    } else {
        int iresult = int(PyInt_AS_LONG(result));
        Py_DECREF(result);
        return iresult;
    }
}
#endif
''',

        # __contains__
        "sq_contains" : '''
static int
%(wrapper_name)s (%(py_struct)s *py_self, PyObject *py_val)
{
    PyObject* result;
    PyObject *args;

    args = Py_BuildValue("(O)", py_val);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
#if PY_MAJOR_VERSION >= 3
    if (result == NULL || PyLong_Check(result) == 0) {
#else
    if (result == NULL || PyInt_Check(result) == 0) {
#endif
        PyErr_SetString(PyExc_RuntimeError, "Unknown error in attempting to test __contains__.");
        Py_XDECREF(result);
        return -1;
    } else {
#if PY_MAJOR_VERSION >= 3
        int iresult = int(PyLong_AS_LONG(result));
#else
        int iresult = int(PyInt_AS_LONG(result));
#endif
        Py_DECREF(result);
        return iresult;
    }
}

''',

        # __iadd__ (in-place concatenation)
        "sq_inplace_concat" : '''
static PyObject*
%(wrapper_name)s (%(py_struct)s *py_self, %(py_struct)s *py_rhs)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(O)", py_rhs);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    return result;
}

''',

        # __imul__ (in-place repeat)
        "sq_inplace_repeat" : '''
static PyObject*
%(wrapper_name)s (%(py_struct)s *py_self, Py_ssize_t py_i)
{
    PyObject *result;
    PyObject *args;

    args = Py_BuildValue("(i)", py_i);
    result = %(method_name)s(py_self, args, NULL);
    Py_DECREF(args);
    return result;
}

''',

        }

    def __init__(self):
        self.slots = {}

    def generate(self, code_sink):
        """
        Generates the structure.  All slots are optional except 'variable'.
        """

        slots = dict(self.slots)

        slots.setdefault('sq_length', 'NULL')
        slots.setdefault('sq_concat', 'NULL')
        slots.setdefault('sq_repeat', 'NULL')
        slots.setdefault('sq_item', 'NULL')
        slots.setdefault('sq_slice', 'NULL')
        slots.setdefault('sq_ass_item', 'NULL')
        slots.setdefault('sq_ass_slice', 'NULL')
        slots.setdefault('sq_contains', 'NULL')
        slots.setdefault('sq_inplace_concat', 'NULL')
        slots.setdefault('sq_inplace_repeat', 'NULL')

        code_sink.writeln(self.TEMPLATE % slots)




import pybindgen.typehandlers.base as typehandlers
from pybindgen.typehandlers import stringtype, ctypeparser
import pybindgen.typehandlers.codesink as codesink
from pybindgen import module, cppclass, overloading, utils
    

import unittest
import doctest
import re


class SmartPointerTransformation(typehandlers.TypeTransformation):
    def __init__(self):
        self.rx = re.compile(r'(?:::)?MySmartPointer<\s*(\w+)\s*>')

    def get_untransformed_name(self, name):
        m = self.rx.match(name)
        if m is None:
            return None
        else:
            return m.group(1)+' *'

    def create_type_handler(self, type_handler, *args, **kwargs):
        ctype = self.get_untransformed_name(args[0])
        handler = type_handler(ctype, *args[1:], **kwargs)
        handler.has_been_transformed = True
        return handler


typehandlers.param_type_matcher.register_transformation(
    SmartPointerTransformation())

class TestParam(typehandlers.Parameter):

    DIRECTIONS = [typehandlers.Parameter.DIRECTION_IN]
    CTYPES = ['testtype*']
    has_been_transformed = False

    def convert_c_to_python(self, wrapper):
        assert isinstance(wrapper, typehandlers.ReverseWrapperBase)
        wrapper.build_params.add_parameter('i', [self.name])

    def convert_python_to_c(self, wrapper):
        assert isinstance(wrapper, typehandlers.ForwardWrapperBase)
        name = wrapper.declarations.declare_variable(self.ctype, self.name)
        wrapper.parse_params.add_parameter('i', ['&'+name], self.name)
        wrapper.call_params.append(name)


class ParamLookupTests(unittest.TestCase):

    def testLookup(self):
        handler = typehandlers.Parameter.new('testtype*', 'name')
        self.assert_(isinstance(handler, TestParam))
        self.assertRaises(typehandlers.TypeLookupError, typehandlers.Parameter.new, 'non_existent_type', 'name')

    def testLookupTransformed(self):
        transformed = typehandlers.Parameter.new('MySmartPointer<testtype>', 'name')
        self.assert_(isinstance(transformed, TestParam))
        self.assert_(transformed.has_been_transformed)
        


if __name__ == '__main__':
    suite = unittest.TestSuite()
    for mod in [
        typehandlers,
        codesink,
        module,
        cppclass,
        overloading,
        #utils,
        stringtype,
        ctypeparser,
        ]:
        suite.addTest(doctest.DocTestSuite(mod))

    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ParamLookupTests))
    runner = unittest.TextTestRunner()
    runner.run(suite)




import sys
sys.path.insert(0, "../../build/examples/buffer")
import c

print c.GetBufferLen()
print c.GetBufferChecksum()
buf = c.GetBuffer()
buf[10] = chr(123)
print c.GetBufferChecksum()
print buf[10]



#
# ----- boost::shared_ptr -----------
#

from .base import Parameter, ReturnValue, \
    join_ctype_and_name, CodeGenerationError, \
    param_type_matcher, return_type_matcher, CodegenErrorBase, \
    DeclarationsScope, CodeBlock, NotSupportedError, ForwardWrapperBase, ReverseWrapperBase, \
    TypeConfigurationError

from pybindgen.cppclass import SmartPointerPolicy, CppClass, CppClassParameterBase, CppClassReturnValueBase, common_shared_object_return

class BoostSharedPtr(SmartPointerPolicy):
    def __init__(self, class_name):
        """
        Create a memory policy for using boost::shared_ptr<> to manage instances of this object.

        :param class_name: the full name of the class, e.g. foo::Bar
        """
        self.class_name = class_name
        self.pointer_template = '::boost::shared_ptr< %s >'

    def get_pointer_name(self, class_name):
        return self.pointer_template % (class_name,)
        
    def get_delete_code(self, cpp_class):
        return "self->obj.~shared_ptr< %s >();" % (cpp_class.full_name,)

    def get_pointer_type(self, class_full_name):
        return self.get_pointer_name(class_full_name) + ' '

    def get_pointer_to_void_name(self, object_name):
        return "%s.get()" % object_name

    def get_instance_creation_function(self):
        return boost_shared_ptr_instance_creation_function

    def get_pystruct_init_code(self, cpp_class, obj):
        return "new(&%s->obj) %s;" % (obj, self.get_pointer_name(cpp_class.full_name),)

    def register_ptr_parameter_and_return(self, cls, name):
        class ThisClassSharedPtrParameter(CppClassSharedPtrParameter):
            """Register this C++ class as pass-by-pointer parameter"""
            CTYPES = []
            cpp_class = cls
        cls.ThisClassSharedPtrParameter = ThisClassSharedPtrParameter
        try:
            param_type_matcher.register(self.get_pointer_name(cls.full_name), cls.ThisClassSharedPtrParameter)
        except ValueError:
            pass

        class ThisClassSharedPtrReturn(CppClassSharedPtrReturnValue):
            """Register this C++ class as pointer return"""
            CTYPES = []
            cpp_class = cls
        cls.ThisClassSharedPtrReturn = ThisClassSharedPtrReturn
        try:
            return_type_matcher.register(self.get_pointer_name(cls.full_name), cls.ThisClassSharedPtrReturn)
        except ValueError:
            pass

    def register_ptr_alias_parameter_and_return(self, cls, alias):        
        alias_ptr = self.get_pointer_name(alias)
        cls.ThisClassSharedPtrParameter.CTYPES.append(alias_ptr)
        try:
            param_type_matcher.register(alias_ptr, cls.ThisClassSharedPtrParameter)
        except ValueError: pass

        cls.ThisClassSharedPtrReturn.CTYPES.append(alias_ptr)
        try:
            return_type_matcher.register(alias_ptr, cls.ThisClassSharedPtrReturn)
        except ValueError: pass

def boost_shared_ptr_instance_creation_function(cpp_class, code_block, lvalue,
                                                parameters, construct_type_name):
    """
    boost::shared_ptr "instance creation function"; it is called whenever a new
    C++ class instance needs to be created

    :param cpp_class: the CppClass object whose instance is to be created
    :param code_block: CodeBlock object on which the instance creation code should be generated
    :param lvalue: lvalue expression that should hold the result in the end
    :param parameters: stringified list of parameters
    :param construct_type_name: actual name of type to be constructed (it is
                          not always the class name, sometimes it's
                          the python helper class)
    """
    assert lvalue
    assert not lvalue.startswith('None')
    if cpp_class.incomplete_type:
        raise CodeGenerationError("%s cannot be constructed (incomplete type)"
                                  % cpp_class.full_name)
    code_block.write_code(
        "%s.reset (new %s(%s));" % (lvalue, construct_type_name, parameters))

        
class StdSharedPtr(BoostSharedPtr):
    def __init__(self, class_name):
        """
        Create a memory policy for using std::shared_ptr<> to manage instances of this object.

        :param class_name: the full name of the class, e.g. foo::Bar
        """
        self.class_name = class_name
        self.pointer_template = '::std::shared_ptr< %s >'

    def get_instance_creation_function(self):
        return std_shared_ptr_instance_creation_function

def std_shared_ptr_instance_creation_function(cpp_class, code_block, lvalue,
                                              parameters, construct_type_name):
    """
    std::shared_ptr "instance creation function"; it is called whenever a new
    C++ class instance needs to be created

    :param cpp_class: the CppClass object whose instance is to be created
    :param code_block: CodeBlock object on which the instance creation code should be generated
    :param lvalue: lvalue expression that should hold the result in the end
    :param parameters: stringified list of parameters
    :param construct_type_name: actual name of type to be constructed (it is
                          not always the class name, sometimes it's
                          the python helper class)
    """
    assert lvalue
    assert not lvalue.startswith('None')
    if cpp_class.incomplete_type:
        raise CodeGenerationError("%s cannot be constructed (incomplete type)"
                                  % cpp_class.full_name)
    code_block.write_code(
        "%s = std::make_shared<%s>(%s);" % (lvalue, construct_type_name, parameters))


class CppClassSharedPtrParameter(CppClassParameterBase):
    "Class* handlers"
    CTYPES = []
    cpp_class = None #cppclass.CppClass('dummy') # CppClass instance
    DIRECTIONS = [Parameter.DIRECTION_IN,
                  Parameter.DIRECTION_OUT,
                  Parameter.DIRECTION_INOUT]
    SUPPORTS_TRANSFORMATIONS = False

    def __init__(self, ctype, name, direction=Parameter.DIRECTION_IN, is_const=False,
                 null_ok=False, default_value=None):
        """
        Type handler for a pointer-to-class parameter (MyClass*)

        :param ctype: C type, normally 'MyClass*'
        :param name: parameter name

        :param is_const: if true, the parameter has a const attached to the leftmost

        :param null_ok: if true, None is accepted and mapped into a C NULL pointer

        :param default_value: default parameter value (as C expression
            string); probably, the only default value that makes sense
            here is probably 'NULL'.

        .. note::

            Only arguments which are instances of C++ classes
            wrapped by PyBindGen can be used as custodians.
        """
        super(CppClassSharedPtrParameter, self).__init__(
            ctype, name, direction, is_const, default_value)
        self.null_ok = null_ok


    def convert_python_to_c(self, wrapper):
        "parses python args to get C++ value"
        assert isinstance(wrapper, ForwardWrapperBase)
        assert isinstance(self.cpp_class, CppClass)

        self.py_name = wrapper.declarations.declare_variable(
            self.cpp_class.pystruct+'*', self.name,
            initializer=(self.default_value and 'NULL' or None))

        value_ptr = wrapper.declarations.declare_variable(
            self.cpp_class.memory_policy.get_pointer_name(self.cpp_class.full_name), "%s_ptr" % self.name)

        if self.null_ok:
            num = wrapper.parse_params.add_parameter('O', ['&'+self.py_name], self.name, optional=bool(self.default_value))

            wrapper.before_call.write_error_check(

                "%s && ((PyObject *) %s != Py_None) && !PyObject_IsInstance((PyObject *) %s, (PyObject *) &%s)"
                % (self.py_name, self.py_name, self.py_name, self.cpp_class.pytypestruct),

                'PyErr_SetString(PyExc_TypeError, "Parameter %i must be of type %s");' % (num, self.cpp_class.name))

            wrapper.before_call.write_code("if (%(PYNAME)s) {\n"
                                           "    if ((PyObject *) %(PYNAME)s == Py_None)\n"
                                           "        %(VALUE)s = NULL;\n"
                                           "    else\n"
                                           "        %(VALUE)s = %(PYNAME)s->obj;\n"
                                           "} else {\n"
                                           "    %(VALUE)s = NULL;\n"
                                           "}" % dict(PYNAME=self.py_name, VALUE=value_ptr))

        else:

            wrapper.parse_params.add_parameter(
                'O!', ['&'+self.cpp_class.pytypestruct, '&'+self.py_name], self.name, optional=bool(self.default_value))
            wrapper.before_call.write_code("if (%s) { %s = %s->obj; }" % (self.py_name, value_ptr, self.py_name))

        wrapper.call_params.append(value_ptr)
        


    def convert_c_to_python(self, wrapper):
        """foo"""

        ## Value transformations
        value = self.transformation.untransform(
            self, wrapper.declarations, wrapper.after_call, self.value)

        ## declare wrapper variable
        py_name = wrapper.declarations.declare_variable(
            self.cpp_class.pystruct+'*', 'py_'+self.cpp_class.name)
        self.py_name = py_name

        def write_create_new_wrapper():
            """Code path that creates a new wrapper for the parameter"""

            ## Find out what Python wrapper to use, in case
            ## automatic_type_narrowing is active and we are not forced to
            ## make a copy of the object
            if self.cpp_class.automatic_type_narrowing:

                typeid_map_name = self.cpp_class.get_type_narrowing_root().typeid_map_name
                wrapper_type = wrapper.declarations.declare_variable(
                    'PyTypeObject*', 'wrapper_type', '0')
                wrapper.before_call.write_code(
                    '%s = %s.lookup_wrapper(typeid(*%s), &%s);'
                    % (wrapper_type, typeid_map_name, value, self.cpp_class.pytypestruct))
            else:
                wrapper_type = '&'+self.cpp_class.pytypestruct

            ## Create the Python wrapper object
            self.cpp_class.write_allocate_pystruct(wrapper.before_call, py_name, wrapper_type)
            self.py_name = py_name

            wrapper.before_call.write_code("%s->flags = PYBINDGEN_WRAPPER_FLAG_NONE;" % py_name)

            ## Assign the C++ value to the Python wrapper
            wrapper.before_call.write_code("%s->obj = %s;" % (py_name, value))

        if self.cpp_class.helper_class is None:
            try:
                self.cpp_class.wrapper_registry.write_lookup_wrapper(
                    wrapper.before_call, self.cpp_class.pystruct, py_name, value)
            except NotSupportedError:
                write_create_new_wrapper()
                self.cpp_class.wrapper_registry.write_register_new_wrapper(wrapper.before_call, py_name,
                                                                           "%s->obj" % py_name)
            else:
                wrapper.before_call.write_code("if (%s == NULL)\n{" % py_name)
                wrapper.before_call.indent()
                write_create_new_wrapper()
                self.cpp_class.wrapper_registry.write_register_new_wrapper(wrapper.before_call, py_name,
                                                                           "%s->obj" % py_name)
                wrapper.before_call.unindent()
                wrapper.before_call.write_code('}')
            wrapper.build_params.add_parameter("N", [py_name])
        else:
            wrapper.before_call.write_code("if (typeid(*(%s)).name() == typeid(%s).name())\n{"
                                          % (value, self.cpp_class.helper_class.name))
            wrapper.before_call.indent()

            if self.type_traits.target_is_const:
                wrapper.before_call.write_code(
                    "%s = (%s*) (((%s*) ((%s*) %s))->m_pyself);"
                    % (py_name, self.cpp_class.pystruct,
                       self.cpp_class.helper_class.name, self.cpp_class.full_name, value))
                wrapper.before_call.write_code("%s->obj =  (%s*) (%s);" %
                                               (py_name, self.cpp_class.full_name, value))
            else:
                wrapper.before_call.write_code(
                    "%s = (%s*) (((%s*) %s)->m_pyself);"
                    % (py_name, self.cpp_class.pystruct,
                       self.cpp_class.helper_class.name, value))
                wrapper.before_call.write_code("%s->obj = %s;" % (py_name, value))
            wrapper.before_call.write_code("Py_INCREF(%s);" % py_name)
            wrapper.before_call.unindent()
            wrapper.before_call.write_code("} else {")
            wrapper.before_call.indent()

            try:
                self.cpp_class.wrapper_registry.write_lookup_wrapper(
                    wrapper.before_call, self.cpp_class.pystruct, py_name, value)
            except NotSupportedError:
                write_create_new_wrapper()
                self.cpp_class.wrapper_registry.write_register_new_wrapper(
                    wrapper.before_call, py_name, "%s->obj" % py_name)
            else:
                wrapper.before_call.write_code("if (%s == NULL)\n{" % py_name)
                wrapper.before_call.indent()
                write_create_new_wrapper()
                self.cpp_class.wrapper_registry.write_register_new_wrapper(wrapper.before_call, py_name,
                                                                           "%s->obj" % py_name)
                wrapper.before_call.unindent()
                wrapper.before_call.write_code('}') # closes if (%s == NULL)

            wrapper.before_call.unindent()
            wrapper.before_call.write_code("}") # closes if (typeid(*(%s)) == typeid(%s))\n{
            wrapper.build_params.add_parameter("N", [py_name])
            



class CppClassSharedPtrReturnValue(CppClassReturnValueBase):
    "Class* return handler"
    CTYPES = []
    SUPPORTS_TRANSFORMATIONS = True
    cpp_class = None #cppclass.CppClass('dummy') # CppClass instance

    def __init__(self, ctype, is_const=False):
        """
        :param ctype: C type, normally 'MyClass*'
        """
        super(CppClassSharedPtrReturnValue, self).__init__(ctype, is_const=is_const)

    def get_c_error_return(self): # only used in reverse wrappers
        """See ReturnValue.get_c_error_return"""
        return "return NULL;"

    def convert_c_to_python(self, wrapper):
        """See ReturnValue.convert_c_to_python"""

        ## Value transformations
        value = self.transformation.untransform(
            self, wrapper.declarations, wrapper.after_call, self.value)
        
        # if value is NULL, return None
        wrapper.after_call.write_code("if (!(%s)) {\n"
                                      "    Py_INCREF(Py_None);\n"
                                      "    return Py_None;\n"
                                      "}" % value)

        ## declare wrapper variable
        py_name = wrapper.declarations.declare_variable(
            self.cpp_class.pystruct+'*', 'py_'+self.cpp_class.name)
        self.py_name = py_name

        common_shared_object_return(value, py_name, self.cpp_class, wrapper.after_call,
                                    self.type_traits, caller_owns_return=True,
                                    reference_existing_object=False,
                                    type_is_pointer=True)

        # return the value
        wrapper.build_params.add_parameter("N", [py_name], prepend=True)
    

    def convert_python_to_c(self, wrapper):
        """See ReturnValue.convert_python_to_c"""
        name = wrapper.declarations.declare_variable(
            self.cpp_class.pystruct+'*', "tmp_%s" % self.cpp_class.name)
        wrapper.parse_params.add_parameter(
            'O!', ['&'+self.cpp_class.pytypestruct, '&'+name])

        value = self.transformation.transform(
            self, wrapper.declarations, wrapper.after_call, "%s->obj" % name)

        # caller gets a shared pointer
        wrapper.after_call.write_code("%s = %s;" % (self.value, value))




#! /usr/bin/env python

import sys

from pybindgen import FileCodeSink
from pybindgen.gccxmlparser import ModuleParser

def my_module_gen():
    module_parser = ModuleParser('a2', '::')
    module_parser.parse([sys.argv[1]], includes=['"a.h"'], pygen_sink=FileCodeSink(sys.stdout))

if __name__ == '__main__':
    my_module_gen()
